/* ------------------------------------------------------------------------- */
/*
 *  mhttpclient.cpp
 *
 *  Copyright (c) 2004 - 2008, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Wed 16 Jul 2008 18:50:00 JST
 */
/* ------------------------------------------------------------------------- */
#include <iostream>
#include <string>
#include "msocket.h"
#include "clx/tokenizer.h"
#include "clx/argument.h"

int main(int argc, char* argv[]) {
	clx::argument arg(argc, argv);
	if (arg.head().empty()) {
		std::cerr << "usage mhttpclient URL [-i interval]" << std::endl;
		std::exit(-1);
	}
	
	clx::format_separator<char> f("http://%s/%s");
	clx::strftokenizer token(arg.head().at(0), f);
	if (token.size() < 2) std::exit(-1);
	
	double interval = -1.0;
	arg("i,interval", interval);
	
	try {
		clx::tcp::sockmanager sm;
		
		measure_socket* clt = new measure_socket(token.at(0), 80);
		std::string query = "GET /" + token.at(1) + " HTTP/1.0\r\n\r\n";
		clt->send(query);
		
		sm.add(clt, measure_handler());
		
		while (1) {
			sm.start(interval);
			if (sm.empty()) break;
			else clt->putrate();
		}
	}
	catch (clx::socket_error& e) {
		std::cerr << e.what() << std::endl;
		std::exit(-1);
	}
	catch (clx::sockaddress_error& e) {
		std::cerr << e.what() << std::endl;
		std::exit(-1);
	}
	
	return 0;
}
