/* ------------------------------------------------------------------------- */
/*
 *  msocket.h
 *
 *  Copyright (c) 2004 - 2008, clown. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    - No names of its contributors may be used to endorse or promote
 *      products derived from this software without specific prior written
 *      permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  Last-modified: Tue 15 Jul 2008 05:04:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef MSOCKET_H
#define MSOCKET_H

#include <cstring>
#include "clx/tcp.h"
#include "clx/sockhandler.h"
#include "clx/format.h"

/* ------------------------------------------------------------------------- */
/*
 *  measure_socket
 *
 *  The class uses when we measure the TCP throughput.
 */
/* ------------------------------------------------------------------------- */
class measure_socket : public clx::tcp::socket {
public:
	measure_socket() : super(), t_(), bytes_(0.0){}
	
	measure_socket(const clx::tcp::socket& cp) :
		super(cp), t_(), bytes_(0.0), bytes_prev_(0.0) {
		this->initialize();
	}
	
	measure_socket(const std::string& host, int port) :
		super(host, port), t_(), bytes_(0.0), bytes_prev_(0.0) {
		this->initialize();
	}
	
	virtual ~measure_socket() {}
	
	void add(double bytes) { bytes_ += bytes; }
	
	void initialize() {
		std::cout << clx::format("%s:%d establish")
			% this->address().ipaddr() % this->address().port()
		<< std::endl;
	}
	
	void terminate() {
		std::cout << clx::format("%s:%d close ( %.2lf sec %.3lf Mbytes %.3lf Mbps )")
			% this->address().ipaddr() % this->address().port()
			% t_.total_elapsed() % bytes_ % (bytes_ / t_.total_elapsed() * 8)
		<< std::endl;
		this->close();
	}
	
	template <class OutputStream>
	void putrate(OutputStream& sout) {
		sout << clx::format("%.2lf sec %7.3lf Mbps ( from %s:%d )")
			% t_.total_elapsed() % ((bytes_ - bytes_prev_) / t_.elapsed() * 8)
			% this->address().ipaddr() % this->address().port()
		<< std::endl;
		bytes_prev_ = bytes_;
		t_.update();
	}
	
	void putrate() { this->putrate(std::cout); }
	
private:
	typedef clx::tcp::socket super;
	
	clx::timer t_;
	double bytes_;		// MByte
	double bytes_prev_;	// MByte
};

/* ------------------------------------------------------------------------- */
//  measure_handler
/* ------------------------------------------------------------------------- */
class measure_handler {
public:
	typedef measure_socket socket_type;
	enum { Mbyte = 1000000 };
	
	measure_handler() : accid_(-1) {}
	measure_handler(socket_int s) : accid_(s) {}
	
	template <class T, class SockManager>
	bool operator()(T* s, SockManager& sm) {
		socket_type* sock = dynamic_cast<socket_type*>(s);
		
		char buf[Mbyte];
		std::memset(buf, 0, sizeof(buf));
		int len = sock->recv(buf, sizeof(buf));
		if (len <= 0) {
			sock->terminate();
			return false;
		}
		sock->add(len / static_cast<double>(Mbyte));
		return true;
	}
	
private:
	socket_int accid_;
};

typedef clx::tcp::accept_handler<measure_socket, measure_handler> accept_handler;

#endif // MSOCKET_H
